searchData={"items":[{"type":"module","title":"Kale","doc":"An `ExUnit` test case for writing features using Kale.\n\n# Usage\n\n```elixir\ndefmodule MyTest do\n  use ExUnit.Case, async: true\n  use Kale\n\n  feature \"Feature description here\" do\n    scenario \"Scenario description here\", \"\"\"\n    Given some precondition\n    When action {foo} happens\n    Then the result is {bar}\n    \"\"\"\n  end\n\n  defgiven \"some precondition\" do\n    # ...\n  end\n\n  defwhen \"action {action} happens\" do\n    # interpolated variables are magically available\n    result = do_something(action)\n    # if the step returns a {:reply, _} tuple containing a map or keyword\n    # list, it will be merged with the test context\n    {:reply, result: result}\n  end\n\n  defthen \"the result is {expected}\", %{result: result} do\n    # Optional second argument for context, as per standard ExUnit\n    assert result == expected\n  end\nend\n```\n\nEach `feature` generates a `describe`, and each `scenario` a `test`. Standard\nExUnit features such as `setup` can be used as normal.\n\nTo avoid the formatter inserting extra parens, you can specify `import_deps:\n[:kale]` in your `.formatter.exs`.","ref":"Kale.html"},{"type":"module","title":"Kale.Macros","doc":"Macros, automatically imported by `use Kale`.","ref":"Kale.Macros.html"},{"type":"macro","title":"Kale.Macros.defgiven/3","doc":"Generate a step definition matching a particular string and optionally a\ncontext map. See the module documentation for usage examples.\n\nThe given, when and then steps are actually interchangeable &ndash; the\nseparate macros are provided for readability only.","ref":"Kale.Macros.html#defgiven/3"},{"type":"macro","title":"Kale.Macros.defthen/3","doc":"An alias for `defgiven/2`.","ref":"Kale.Macros.html#defthen/3"},{"type":"macro","title":"Kale.Macros.defwhen/3","doc":"An alias for `defgiven/2`.","ref":"Kale.Macros.html#defwhen/3"},{"type":"macro","title":"Kale.Macros.feature/2","doc":"Generate a feature block, which corresponds to an ExUnit `describe`.","ref":"Kale.Macros.html#feature/2"},{"type":"macro","title":"Kale.Macros.scenario/2","doc":"Generate a scenario block, which corresponds to an ExUnit `test`.","ref":"Kale.Macros.html#scenario/2"},{"type":"module","title":"Kale.Utils","doc":"Utility functions, used by macros and the functions they create.","ref":"Kale.Utils.html"},{"type":"function","title":"Kale.Utils.extract_args/1","doc":"Return the interpolated variables or argument names from a string. Elements\nin the returned list will always be strings.\n\n```\niex> Kale.Utils.extract_args(\"When I add {a} and{b}\")\n[\"a\", \"b\"]\n\niex> Kale.Utils.extract_args(\"When I add {1} and{2}\")\n[\"1\", \"2\"]\n```","ref":"Kale.Utils.html#extract_args/1"},{"type":"function","title":"Kale.Utils.normalise_name/1","doc":"Remove interpolated variables from a step string, so that the call matches\nthe definition.\n\n```\niex> Kale.Utils.normalise_name(\"I log on as {username}\")\n\"I log on as {}\"\n\niex> Kale.Utils.normalise_name(\"I log on as {fred}\")\n\"I log on as {}\"\n```","ref":"Kale.Utils.html#normalise_name/1"},{"type":"extras","title":"Kale","doc":"# Kale\n\nBasic given-when-then steps for ExUnit tests.","ref":"readme.html"},{"type":"extras","title":"Why? - Kale","doc":"I have previously used [White Bread](https://github.com/meadsteve/white-bread)\nand [Cabbage](https://github.com/cabbage-ex/cabbage) for writing Cucumber-style\n(Gherkin) tests, but decided I didn't really need the extra complexity over\nsimple ExUnit tests. However, I quite like writing acceptance tests in a\n\"given, when, then\" format &ndash; I find it makes the described functionality\neasier to read, even if you aren't going the full BDD route and writing the\nscenarios with your customers.\n\nThis library is an experiment to try to come up with a minimal layer on top of\nExUnit to allow this. It was also an excuse to delve into the world of Elixir\nmetaprogramming.\n\nKale does **not** implement the full gherkin language. Instead it uses simple\nstring matching (no regexes) with the ability to explicitly interpolate values\nby surrounding them with `{}`.","ref":"readme.html#why"},{"type":"extras","title":"Why \"Kale\"? - Kale","doc":"As a nod to [Spinach](https://github.com/codegram/spinach).","ref":"readme.html#why-kale"},{"type":"extras","title":"Installation - Kale","doc":"Kale is [available in Hex](https://hex.pm/packages/kale), and can be installed\nby adding it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kale, \"~> 0.8.0\", only: :test}\n  ]\nend\n```\n\nThen run `mix deps.get`.\n\nIf you want to use Kale's formatter rules (to avoid having parens inserted\naround the arguments of `scenario`, `defwhen` etc), import it in your\n`.formatter.exs`:\n\n```elixir\n[\n  # ... \n  import_deps: [:kale]\n]\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Kale","doc":"In your test, `use Kale`:\n\n```elixir\ndefmodule MyTest do\n  use ExUnit.Case, async: true\n  use Kale\n\n  # ...\nend\n```\n\nThen write a `feature` (which corresponds to an ExUnit `describe`) containing\none or more scenarios:\n\n```elixir\nfeature \"Feature description here\" do\n\n  scenario \"Scenario description here\", \"\"\"\n  Given some precondition\n  When action {foo} happens\n  Then the result is {bar}\n  \"\"\"\n\n  scenario \"Another scenario\", \"\"\"\n  ...\n  \"\"\"\nend\n```\n\nEach scenario has a description, and a multiline string containing a list of\nsteps, which should all begin with one of `Given`, `When`, `Then`, `And` or\n`But`. Lines beginning with anything else are treated as comments and ignored.\n\nEach unique step needs a definition, which needs to be created in the test\nmodule by calling `defgiven`, `defwhen` or `defthen`. Technically it doesn't\nmatter which of the three you use, but it's recommended to use the one that\nmatches the step for redability.\n\nThe simplest form of step definition just takes a literal string argument\nmatching the step from the scenario, and a block of code to execute:\n\n```elixir\ndefgiven \"some precondition\" do\n  # ...\nend\n```\n\nIf you need to access the ExUnit context, it can be provided as a second\nargument:\n\n```elixir\ndefgiven \"some precondition\", context do\n  login(context.user)\nend\n```\n\nYou can also pattern match specific values from the context:\n\n```elixir\ndefgiven \"some precondition\", %{user: user} do\n  login(user)\nend\n```\n\nIf a step definition returns a `{:reply, context}` tuple, where `context` is a\nmap or keyword list, it will be merged into the context for future steps in the\nsame scenario. For example, to store a value for `session`:\n\n```elixir\ndefgiven \"some precondition\", %{user: user} do\n  session = login(context.user)\n  {:reply, session: session}\nend\n```\n\nValues can be interpolated by wrapping them in `{}`. The value from the step in\nthe scanario will be available within the body of the step definition using the\nvariable name provided:\n\n```elixir\ndefwhen \"action {action} happens\" do\n  do_something(action)\nend\n```\n\nTests are run just like normal ExUnit tests (which is what they compile to).\nThis includes being able to specify the line number of the scenario or feature.\n\nTests are reported as \"features\" by ExUnit, separately from other tests:\n\n```\n$ mix test\n................\n\nFinished in 0.08 seconds\n6 features, 10 tests, 0 failures\n\nRandomized with seed 955579\n```","ref":"readme.html#usage"},{"type":"extras","title":"Development - Kale","doc":"To build the project after cloning:\n\n```bash\n$ mix deps.get\n$ make\n```","ref":"readme.html#development"},{"type":"extras","title":"TODO - Kale","doc":"* More helpful errors when no step matches","ref":"readme.html#todo"},{"type":"extras","title":"LICENSE","doc":"","ref":"license.html"},{"type":"extras","title":"MIT License - LICENSE","doc":"**Copyright (c) 2019–2024 Kerry Buckley**\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html#mit-license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.8.1] - 2024-06-27","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- Switch to ExUnit.Case.register_test/6 to address deprecation warning\n\n## [0.8.0] - 2019-06-13","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Only update context if step explicitly returns a `{:reply, context}` tuple\n\n## [0.7.0] - 2019-06-12","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove `FeatureCase`. Clients should `use Kale` as well as the normal `ExUnit.Case` etc.\n\n## [0.6.2] - 2019-06-10","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fix hex link in README\n\n## [0.6.1] - 2019-06-10","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fix some typos in the README\n\n## [0.6.0] - 2019-06-10","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Report tests as \"features\" when running `mix test`\n- Allow keyword lists and `:ok` tuples for returning context (not just maps)\n- Set minimum Elixir version to 1.3 (when `describe` was added to ExUnit)\n- Link to source code from docs\n- Add licence to docs\n- Add this changelog\n\n## [0.5.1] - 2019-06-07","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Improved documentation\n- More checks, and a makefile\n\n## [0.5.0] - 2019-06-06","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Automatically make interpolated variables available in steps\n- Make context argument optional\n\n## [0.4.0] - 2019-06-05","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Specify steps as multiline strings\n- Pass context between steps\n\n## [0.3.0] - 2019-06-04","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Make `args` and `context` explicit arguments of `defwhen` etc\n\n## [0.2.0] - 2019-05-27","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Move `FeatureCase` into `Kale` namespace\n\n## [0.1.1] - 2019-05-27","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Include README in generated docs\n\n## [0.1.0] - 2019-05-27","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release\n\n[Unreleased]: https://github.com/kerryb/kale/compare/0.8.0...HEAD\n[0.8.0]: https://github.com/kerryb/kale/compare/0.7.0...0.8.0\n[0.7.0]: https://github.com/kerryb/kale/compare/0.6.2...0.7.0\n[0.6.2]: https://github.com/kerryb/kale/compare/0.6.1...0.6.2\n[0.6.1]: https://github.com/kerryb/kale/compare/0.6.0...0.6.1\n[0.6.0]: https://github.com/kerryb/kale/compare/0.5.1...0.6.0\n[0.5.1]: https://github.com/kerryb/kale/compare/0.5.0...0.5.1\n[0.5.0]: https://github.com/kerryb/kale/compare/0.4.0...0.5.0\n[0.4.0]: https://github.com/kerryb/kale/compare/0.3.0...0.4.0\n[0.3.0]: https://github.com/kerryb/kale/compare/0.2.0...0.3.0\n[0.2.0]: https://github.com/kerryb/kale/compare/0.1.1...0.2.0\n[0.1.1]: https://github.com/kerryb/kale/compare/0.1.0...0.1.1\n[0.1.0]: https://github.com/kerryb/kale/releases/tag/0.1.0","ref":"changelog.html#added"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}